import argparse
import logging

import jacovirt.logger

log_level = {
    "CRITICAL": logging.CRITICAL,
    "ERROR": logging.ERROR,
    "WARN": logging.WARNING,
    "INFO": logging.INFO,
    "DEBUG": logging.DEBUG,
}

def main():

    # General parser
    parser = argparse.ArgumentParser(description=__doc__)

    # Subparser
    sub = parser.add_subparsers(dest="sub")

    parser.add_argument(
        "-l",
        "--log",
        choices=log_level.keys(),
        default="WARN",
        help=("Set level log"),
        dest="log",
    )


    # Parser for JacoVirt Pad
    pad = sub.add_parser("pad", help="Run JacoVirt Pad")

    # Parser for JacoVirt Bot
    bot = sub.add_parser("bot", help="Run JacoVirt Bot")

    args = parser.parse_args()

    if args.sub is None:
        parser.print_help()
        exit(1)

    logging.getLogger().setLevel(args.log)

    match args.sub:
        case "pad":
            logging.info("Lunch JacoVirt Pad")
            from jacovirt.pad import window
            window.main()
        case "bot":
            logging.info("Lunch JacoVirt Bot")
            from jacovirt.bot import window
            window.main()
        case _:
            SystemExit
